import asyncio

class ControllerCreator:
    def __init__(self, controller_manager, device_manager, stop_event: asyncio.Event):
        self.controller_manager = controller_manager
        self.device_manager = device_manager
        self.stop_event = stop_event

    def create(self, instance):
            try:
                self.controller = instance
                self._set_managers()
            except Exception as e:
                print("An error occurred while creating controller: ", e)

    def _set_managers(self):
        self.controller.device_manager = self.device_manager
        self.controller_manager.add(self.controller)
        self.controller.stop_event = self.stop_event



from abc import ABC, abstractmethod
from connectus.tools.structure.data import DataRequest

class BaseAcquisition(ABC):
    def __init__(self):
        pass

    def run(self) -> list[DataRequest]:
        try:
            response_list = self.controller.device_manager.get([DataRequest(action= 'get_data', device_ids= self.controller.device_ids)])
            resquests = []
            for response in response_list:
                if response.data.collection:
                    resquests = [DataRequest(action= 'update_data', data= response.data)]
            return resquests
        except Exception as e:
            print(f"An error occurred while running controller: {e}")


from ...base import BaseNode
from .configuration import Configuration
import asyncio
import pyvisa

class VISA(BaseNode, Configuration):
    def __init__(self, node_config: dict[str, str], stop_event: asyncio.Event):
        BaseNode.__init__(self, stop_event)
        Configuration.__init__(self, node_config)

    async def connect(self):
        try:
            self.client = pyvisa.ResourceManager(r'C:\Windows\system32\visa64.dll')
            self.device = self.client.open_resource(self.resource_name, timeout=self.timeout, write_termination = self.write_termination, read_termination = self.read_termination)
        except Exception as e:
            print("An error occurred while starting VISA:", e)
    
    async def disconnect(self):
        self.device.close()
        self.client.close()

    async def write(self, data: list[str]):
        try:
            for msg in data:
                self.device.write(msg)
        except Exception as e:
            print("An error occurred while sending data with VISA protocol:", e)

    async def read(self, variable_list: list[str]):
        try:
            value_list = []
            for variable_name in variable_list:
                variable_value = self.device.query_ascii_values(variable_name)
                value_list.append(variable_value)
            return value_list
        except Exception as e:
            print("An error occurred while receiving data with VISA protocol:", e)



from connectus.tools.structure.data import DataRequest
from abc import ABC, abstractmethod

class BaseDispatch(ABC):
    def __init__(self):
        pass

    def run(self, request_list: list[DataRequest]):
        if request_list:
            self.controller.device_manager.set(request_list)